from common.config import Config
import mysql.connector

class DbManager:
  def __init__(self, config: Config):
    self.config = config

    db = self.get_connection()

    # Create tables if they don't exist
    cursor = db.cursor()
    cursor.execute("""
  CREATE TABLE IF NOT EXISTS `movie` (
    id BIGINT NOT NULL AUTO_INCREMENT,
    movie_name VARCHAR(256) NOT NULL,
    tunefind_url VARCHAR(1024),
    is_series BOOLEAN,
    songs MEDIUMTEXT,
    fetch_date BIGINT,
    lock_date BIGINT,
    PRIMARY KEY (id),
    CONSTRAINT movie_name_uq UNIQUE (movie_name)
  );
    """)

    cursor.execute("""
  CREATE TABLE IF NOT EXISTS `music` (
    file_id VARCHAR(256) NOT NULL,
    spot_id VARCHAR(256) NOT NULL,
    codec VARCHAR(5) NOT NULL,
    bitrate VARCHAR(5) NOT NULL,
    caption VARCHAR(1024),
    PRIMARY KEY (spot_id, codec, bitrate)
  );
    """)

    cursor.execute("""
  CREATE TABLE IF NOT EXISTS `bot_user` (
    id BIGINT NOT NULL,
    state VARCHAR(32) NOT NULL,
    quality VARCHAR(10),
    last_update TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
    PRIMARY KEY (id)
  );
    """)

    cursor.execute("""
CREATE TABLE IF NOT EXISTS lyrics (
  spot_id VARCHAR(256) NOT NULL,
  lyrics LONGTEXT NOT NULL,
  PRIMARY KEY (spot_id)
);    
""")
    
    cursor.execute("""
CREATE TABLE IF NOT EXISTS plan (
  id INTEGER NOT NULL AUTO_INCREMENT,
  name VARCHAR(256) NOT NULL,
  duration_days INTEGER NOT NULL,
  price INTEGER NOT NULL,
  discount INTEGER,
  PRIMARY KEY (id)
);
""")

    cursor.execute("""
CREATE TABLE IF NOT EXISTS plan_member (
  user_id BIGINT NOT NULL,
  plan_id INTEGER NOT NULL,
  start_date BIGINT NOT NULL,
  end_date BIGINT NOT NULL,
  PRIMARY KEY (user_id, plan_id)
);
""")

    cursor.execute("""
    CREATE TABLE IF NOT EXISTS `config` (
      id BIGINT NOT NULL,
      sub_required BOOL NOT NULL DEFAULT FALSE,
      channels VARCHAR(1024),
      channel_links VARCHAR(4096),
      plans_enabled BOOL NOT NULL DEFAULT FALSE,
      PRIMARY KEY (id)
    );
      """)
    db.commit()
    cursor.execute("SELECT * FROM config WHERE id = 1;")
    config = cursor.fetchall()
    if not config or not len(config):
      cursor.execute("INSERT INTO config (id) VALUES (1)")

    db.commit()
    db.close()

  def get_connection(self):
    db = mysql.connector.connect(
      host=self.config.db_host,
      port=self.config.db_port,
      user=self.config.db_username,
      password=self.config.db_password,
      database=self.config.db_database
    )
    return db
