from pyrogram.types import Message, CallbackQuery, InlineKeyboardButton, InlineKeyboardMarkup

from data.admin.main_menu import main_menu
from data.admin.ask_broadcast_message import ask_broadcast_message
from data.admin.plans_menu import plans_menu
from services.repo import ServiceRepo
from common.states import States

async def send_stats(update: CallbackQuery, service_repo: ServiceRepo):
  user_count = service_repo.user_service.get_user_count()
  service_repo.user_service.set_state(str(update.from_user.id), States.ADMIN_WATCHING_STATS)
  await update.message.edit_text(
    text=f"""
📊 آمار ربات

👤 تعداد کل کاربران: {user_count["all_users"]}

👤 تعداد کاربران هفته‌ی اخیر: {user_count["last_week_users"]}

👤 تعداد کاربران یک ماه اخیر: {user_count["last_month_users"]}
""",
    reply_markup=InlineKeyboardMarkup([[InlineKeyboardButton(text='بازگشت', callback_data='back')]])
  )

async def handle_message(message: Message, service_repo: ServiceRepo):
  if message.text.startswith('/start'):
    await message.reply_text(
      text=main_menu["text"],
      reply_markup=main_menu["reply_markup"]
    )

async def handle_callback(update: CallbackQuery, service_repo: ServiceRepo):
  data = update.data
  if data == 'stats':
    await send_stats(update, service_repo)
  elif data == 'broadcast':
    service_repo.user_service.set_state(str(update.from_user.id), States.ADMIN_BROADCASTING_MESSAGE)
    await update.message.edit_text(
      text=ask_broadcast_message["text"],
      reply_markup=ask_broadcast_message["reply_markup"]
    )
  elif data == 'manage_plans':
    service_repo.user_service.set_state(str(update.from_user.id), States.ADMIN_MANAGING_PLANS)
    db_conf = service_repo.db_conf_service.get_conf()
    menu = plans_menu(db_conf["plans_enabled"])
    await update.message.edit_text(
      text=menu["text"],
      reply_markup=menu["reply_markup"]
    )
