from pyrogram.client import Client
from pyrogram.types import Message, CallbackQuery, InlineKeyboardButton, InlineKeyboardMarkup

from selenium import webdriver
from selenium.webdriver.common.by import By

from time import sleep
import concurrent.futures
import json

from services.repo import ServiceRepo
from data.main_menu import main_menu
from common.states import States

def get_season(url, episode):
  options = webdriver.ChromeOptions()
  options.add_argument('--headless')
  options.add_argument("start-maximized")
  options.binary_location = "/usr/bin/google-chrome-stable"
  driver = webdriver.Chrome(options=options)
  driver.execute_cdp_cmd('Network.setUserAgentOverride', {'userAgent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/117.0.0.0 Safari/537.36'})
  driver.set_window_size(1024, 768)
  driver.set_page_load_timeout(60)

  print(f'getting episode {url}')
  driver.get(url)
  sleep(1)
  print('page load')

  while True:
    try:
      element = driver.find_elements(
        By.XPATH, '//div//descendant::h4[starts-with(text(),"E")]')[int(episode) - 1]
      driver.execute_script(
        "arguments[0].scrollIntoView({block: 'center'});", element)
      element.click()
      sleep(2)
      break
    except Exception as e:
      print('e1')
      print(e)
      break

  print('scrolled')
  while True:
    try:
      names = driver.find_elements(
        By.XPATH, '//div[@data-rbd-droppable-id="table-body"]//descendant::p')
      artists = driver.find_elements(
        By.XPATH, '//div[@data-rbd-droppable-id="table-body"]//descendant::a/small')
      sleep(1)
      if len(names) > 0:
        break
    except:
      pass
  
  print('found names')
  try:
    element = driver.find_element(
      By.XPATH, '//button[contains(@class,"ShowMore")]//p[text()="Show all tracks"]/parent::button')
    driver.execute_script(
      "arguments[0].scrollIntoView({block: 'center'});", element)
    element.click()
    sleep(1)
  except:
    pass
  
  print('clicked show more')

  album_url = None
  try:
    element = driver.find_element(
      By.XPATH, '/html/body/div[1]/div/div/div[2]/div[5]/div/div[1]/div[2]/div/div[2]/div/div[3]/a'
    )
    if element:
      album_url = element.get_attribute('href')
  except Exception as e:
    pass

  while True:
    try:
      names = driver.find_elements(
        By.XPATH, '//div[@data-rbd-droppable-id="table-body"]//descendant::p')
      artists = driver.find_elements(
        By.XPATH, '//div[@data-rbd-droppable-id="table-body"]//descendant::a/small')
      sleep(1)
      if len(names) > 0:
        break
    except:
      pass

  sleep(1)
  print('got songs')

  result = []
  for artist in artists:
    result.append({ "artist": artist.text })
  for i in range(len(names)):
    result[i]["name"] = names[i].text

  if album_url:
    print(f'Checking {album_url}')
    driver.execute_script("window.open('');")
    print('open window')
    driver.switch_to.window(driver.window_handles[1])
    print('switch to window')
    driver.get(album_url)
    print('get')
    print(driver.current_url)
    driver.close()

  driver.close()
  return result


async def handle_message(bot: Client, message: Message, service_repo: ServiceRepo):
  if message.text == '/start':
    service_repo.user_service.set_state(message.from_user.id, States.STARTED)
    await bot.send_message(
      chat_id=message.chat.id, text=main_menu["text"], reply_markup=main_menu["reply_markup"]
    )
    return

async def handle_callback_query(bot: Client, callback: CallbackQuery, service_repo: ServiceRepo):
  if callback.data == 'back' or not callback.data.startswith('dl_episode_'):
    service_repo.user_service.set_state(callback.from_user.id, States.STARTED)
    await bot.send_message(
      chat_id=callback.message.chat.id, text=main_menu["text"], reply_markup=main_menu["reply_markup"]
    )
    return

  movie = service_repo.movie_service.get_movie(callback.data.split('_')[2].split(':')[0])
  if movie["songs"]:
    cache_data = json.loads(movie["songs"])
  else:
    cache_data = None
  
  season_episodes = callback.data.split(':')
  season = season_episodes[1]
  episode = season_episodes[2]
  url = f'{movie["tunefind_url"]}/season-{season}'
  
  # Check for cache
  if cache_data and cache_data["seasons"] and cache_data["seasons"][int(season) - 1] and cache_data["seasons"][int(season) - 1]["episodes"] and cache_data["seasons"][int(season) - 1]["episodes"][int(episode) - 1]:
    songs = cache_data["seasons"][int(season) - 1]["episodes"][int(episode) - 1]
    reply_markup = []
    for song in songs:
      reply_markup.append([InlineKeyboardButton(
        text=f'{song["name"]} - {song["artist"]}',
        callback_data=f'spotsearch_{song["spot_id"]}'
      )])

    reply_markup.append([InlineKeyboardButton(text='بازگشت', callback_data='back')])
    service_repo.user_service.set_state(callback.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
    await bot.edit_message_text(
      chat_id=callback.message.chat.id,
      message_id=callback.message.id,
      text='آهنگ مورد نظر را انتخاب کنید',
      reply_markup=InlineKeyboardMarkup(reply_markup)
    )
    return

  try:
    with concurrent.futures.ThreadPoolExecutor() as executor:
      future = executor.submit(get_season, url, episode)
      result = future.result(timeout=30)
      if not result or not len(result):
        service_repo.user_service.set_state(callback.from_user.id, States.STARTED)
        await bot.send_message(
          chat_id=callback.message.chat.id, text=f'متاسفانه آهنگی برای فیلم انتخاب شده پیدا نشد\n{main_menu["text"]}', reply_markup=main_menu["reply_markup"]
        )
        return
      
      songs = []
      for song in result:
        spot_info = await service_repo.spotify_service.base_spotify_search(
          f'track:{song["name"]} artist:{song["artist"]}',
          1
        )
        if not spot_info or not spot_info["tracks"] or not spot_info["tracks"]["items"] or not len(spot_info["tracks"]["items"]):
          continue
        spot_id = spot_info["tracks"]["items"][0]["id"]
        songs.append({
          "name": song["name"],
          "artist": song["artist"],
          "spot_id": spot_id
        })

      reply_markup = []
      for song in songs:
        if cache_data and cache_data["seasons"]:
          cache_data["seasons"][int(season) - 1]["episodes"][int(episode) - 1].append({
            "name": song["name"],
            "artist": song["artist"],
            "spot_id": song["spot_id"]
          })
        reply_markup.append([InlineKeyboardButton(
          text=f'{song["name"]} - {song["artist"]}',
          callback_data=f'spotsearch_{song["spot_id"]}'
        )])

      service_repo.movie_service.update_movie(movie["id"], True, json.dumps(cache_data))
      reply_markup.append([InlineKeyboardButton(text='بازگشت', callback_data='back')])
      service_repo.user_service.set_state(callback.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
      await bot.edit_message_text(
        chat_id=callback.message.chat.id,
        message_id=callback.message.id,
        text='آهنگ مورد نظر را انتخاب کنید',
        reply_markup=InlineKeyboardMarkup(reply_markup)
      )
  except Exception as e:
    print(e)
    service_repo.user_service.set_state(callback.from_user.id, States.STARTED)
    await bot.send_message(
      chat_id=callback.message.chat.id, text=f'خطایی رخ داده است، مجددا امتحان کنید\n{main_menu["text"]}', reply_markup=main_menu["reply_markup"]
    )
    return
