from pyrogram.client import Client
from pyrogram.types import Message, CallbackQuery, InlineKeyboardButton, InlineKeyboardMarkup
import concurrent.futures
import json

from services.repo import ServiceRepo
from data.main_menu import main_menu
from common.states import States

async def handle_message(bot: Client, message: Message, service_repo: ServiceRepo):
  if message.text == '/start':
    service_repo.user_service.set_state(message.from_user.id, States.STARTED)
    await bot.send_message(
      chat_id=message.chat.id, text=main_menu["text"], reply_markup=main_menu["reply_markup"]
    )
    return

async def handle_callback_query(bot: Client, callback: CallbackQuery, service_repo: ServiceRepo):
  if callback.data == 'back':
    service_repo.user_service.set_state(callback.from_user.id, States.STARTED)
    await bot.send_message(
      chat_id=callback.message.chat.id, text=main_menu["text"], reply_markup=main_menu["reply_markup"]
    )
    return
  if callback.data.startswith("refresh:"):
    _, movie_name = callback.data.split(":")
    movie = service_repo.movie_service.get_movie(movie_name)
    url = movie["tunefind_url"]
    print(url)
    with concurrent.futures.ThreadPoolExecutor() as executor:
      future = executor.submit(service_repo.movie_service.get_show_info, url)
      result = future.result(timeout=30)
      if not result or not len(result):
        service_repo.user_service.set_state(callback.from_user.id, States.STARTED)
        await bot.send_message(
          chat_id=callback.message.chat.id, text=f'متاسفانه آهنگی برای فیلم انتخاب شده پیدا نشد\n{main_menu["text"]}', reply_markup=main_menu["reply_markup"]
        )
        return

      reply_markup = []
      cache_data = {
        "season_count": len(result),
        "seasons": []
      }
      for i in range(len(result)):
        cache_data["seasons"].append({
          "episode_count": result[i]["episodes"],
          "episodes": [[]] * int(result[i]["episodes"])
        })
        reply_markup.append([
          InlineKeyboardButton(text=result[i]["season"], callback_data=f'dl_season_{movie["movie_name"]}:{i + 1}:{result[i]["episodes"]}')
        ])
      reply_markup.append([InlineKeyboardButton(text='بازگشت', callback_data='back')])
      service_repo.movie_service.update_movie(movie["id"], True, json.dumps(cache_data))
      service_repo.user_service.set_state(callback.from_user.id, States.CHOOSING_SEASON)
      await bot.edit_message_text(
        chat_id=callback.message.chat.id,
        message_id=callback.message.id,
        text=f"""
  [🎬 {movie["movie_name"]}](https://t.me/gemostbot?start={movie["id"]})

  یک فصل را انتخاب کنید
  """,
        reply_markup=InlineKeyboardMarkup(reply_markup)
      )
      return

  movie = service_repo.movie_service.get_movie(callback.data.split('_')[2].split(':')[0])
  season_episodes = callback.data.split(':')
  season = season_episodes[1]
  episodes = int(season_episodes[2])

  service_repo.user_service.set_state(callback.from_user.id, States.CHOOSING_EPISODE)

  reply_markup = []
  for i in range(episodes):
    reply_markup.append([InlineKeyboardButton(
      text=f'قسمت {i + 1}',
      callback_data=f'dl_episode_{movie["movie_name"]}:{season}:{i + 1}'
    )])
  reply_markup.append([InlineKeyboardButton(text='بازگشت', callback_data='back')])
  await bot.send_message(
    chat_id=callback.message.chat.id,
    text=f"""
{movie["movie_name"]} - Season {season}

این فصل دارای {episodes} قسمت است، لطفا شماره قسمت مورد نظر را انتخاب کنید
""",
  reply_markup=InlineKeyboardMarkup(reply_markup)
  )
