from pyrogram.types import Message, CallbackQuery, InlineKeyboardButton, InlineKeyboardMarkup
from pyrogram.client import Client
from services.repo import ServiceRepo
from common.states import States
from common.quality import Quality
from common.config import Config
from data.main_menu import main_menu
from controllers import wait_for_spotify_search as wait_for_spotify_search_controller

async def handle_message(bot: Client, message: Message, service_repo: ServiceRepo, db_conf, app_conf: Config):
  await wait_for_spotify_search_controller.handle_message(bot, message, service_repo, db_conf, app_conf)

async def handle_callback_query(bot: Client, callback: CallbackQuery, service_repo: ServiceRepo, db_conf, app_conf: Config):
  data = callback.data
  user_id = callback.from_user.id
  if data == 'back':
    await bot.send_message(
      chat_id=user_id,
      text=main_menu["text"],
      reply_markup=main_menu["reply_markup"]
    )
    service_repo.user_service.set_state(user_id, States.STARTED)

  elif data.startswith('spotsearch_'):
    _, spot_id = data.split('_')
    quality = service_repo.user_service.get_quality(user_id)
    plan = service_repo.plan_member_service.get_user_plan(user_id)
    if not (user_id in app_conf.admin_user_ids) and db_conf["plans_enabled"] and quality == Quality.FLAC and not plan:
      await callback.message.reply_text(
        text="برای دانلود با بالاترین کیفیت، اشتراک ربات را خریداری کنید. تنظیمات کیفیت دانلود به MP3 320 تغییر یافت"
      )
      quality = Quality.MP3_320
      service_repo.user_service.set_quality(user_id, Quality.MP3_320)

    await service_repo.spotify_service.spotify_dl(
      bot,
      callback.message,
      spot_id,
      None,
      quality,
      service_repo.user_service,
      service_repo.music_service
    )

  elif data.startswith('more:'):
    _, offset, limit, search_term = data.split(':')
    offset = int(offset)
    limit = int(limit)
    await service_repo.spotify_service.spotify_search(bot, callback.message, search_term, callback.message, service_repo.user_service, offset, limit)

  elif data.startswith('lyrics:'):
    _, spot_id = data.split(':')
    song = await service_repo.spotify_service.fetch_spotify_track(spot_id)
    if song:
      lyrics = service_repo.lyrics_service.get_lyrics_via_api(
        song['artist'], song['name']
      )
      if lyrics:
        await callback.message.reply_text(
          text=lyrics,
          reply_markup=InlineKeyboardMarkup([[InlineKeyboardButton(text="بازگشت", callback_data="back")]])
        )
      else:
        await error_handler(callback.message, callback.from_user, "متن آهنگ پیدا نشد", service_repo)
    else:
      await error_handler(callback.message, callback.from_user, "آهنگ پیدا نشد", service_repo)

async def error_handler(message: Message, user_id: int, text: str, service_repo: ServiceRepo):
  service_repo.user_service.set_state(user_id, States.STARTED)
  await message.reply_text(
    text=f"""
{text}
{main_menu["text"]}
""",
    reply_markup=main_menu["reply_markup"]
  )
