from pyrogram.types import Message, CallbackQuery
from pyrogram.client import Client

from common.states import States
from common.config import Config
from services.repo import ServiceRepo
from controllers import null as null_controller
from controllers import started as started_controller
from controllers import choosing_quality as choosing_quality_controller
from controllers import wait_for_spotify_search as wait_for_spotify_search_controller
from controllers import choosing_spotify_result as choosing_spotify_result_controller
from controllers import wait_for_movie_search as wait_for_movie_search_controller
from controllers import choosing_movie as choosing_movie_controller
from controllers import choosing_season as choosing_season_controller
from controllers import choosing_episode as choosing_episode_controller
from controllers import viewing_plan as viewing_plan_controller
from controllers import choosing_plan as choosing_plan_controller
from controllers import viewing_plan_info as viewing_plan_info_controller

from controllers.admin import null as admin_null_controller
from controllers.admin import started as admin_started_controller
from controllers.admin import watching_stats as admin_watching_stats_controller
from controllers.admin import broadcasting_message as admin_broadcasting_message_controller
from controllers.admin import managing_plans as admin_managing_plans_controller
from controllers.admin import viewing_plan_list as admin_viewing_plan_list_controller
from controllers.admin import setting_plan_name as admin_setting_plan_name_controller
from controllers.admin import setting_plan_price as admin_setting_plan_price_controller

async def handle_message(bot: Client, app_conf: Config, message: Message, state: States, service_repo: ServiceRepo, db_conf):
  if message.text == '/start':
    await null_controller.handle_message(bot, message, service_repo)
  elif message.text == '/admin' and message.from_user.id in app_conf.admin_user_ids:
    await admin_null_controller.handle_message(message, service_repo)
  elif state == States.NULL:
    await null_controller.handle_message(bot, message, service_repo)
  elif state == States.STARTED:
    await started_controller.handle_message(bot, message, service_repo, db_conf, app_conf)
  elif state == States.WAIT_FOR_SPOTIFY_SEARCH:
    await wait_for_spotify_search_controller.handle_message(bot, message, service_repo, db_conf, app_conf)
  elif state == States.CHOOSING_SPOTIFY_RESULT:
    await choosing_spotify_result_controller.handle_message(bot, message, service_repo, db_conf, app_conf)
  elif state == States.WAIT_FOR_MOVIE_SEARCH:
    await wait_for_movie_search_controller.handle_message(bot, message, service_repo)
  elif state == States.CHOOSING_MOVIE:
    await choosing_movie_controller.handle_message(bot, message, service_repo)
  elif state == States.CHOOSING_SEASON:
    await choosing_season_controller.handle_message(bot, message, service_repo)
  elif state == States.CHOOSING_EPISODE:
    await choosing_episode_controller.handle_message(bot, message, service_repo)
  elif state == States.VIEWING_PLAN:
    await viewing_plan_controller.handle_message(message, service_repo)
  elif state == States.CHOOSING_PLAN:
    await choosing_plan_controller.handle_message(message, service_repo)
  elif state == States.VIEWING_PLAN_INFO:
    await viewing_plan_info_controller.handle_message(message, service_repo)
  else:
    # Admin states
    # Check if user is admin
    if message.from_user.id in app_conf.admin_user_ids:
      if state == States.ADMIN_STARTED:
        await admin_started_controller.handle_message(message, service_repo)
      elif state == States.ADMIN_WATCHING_STATS:
        await admin_watching_stats_controller.handle_message(message, service_repo)
      elif state == States.ADMIN_BROADCASTING_MESSAGE:
        await admin_broadcasting_message_controller.handle_message(bot, message, service_repo)
      elif state == States.ADMIN_MANAGING_PLANS:
        await admin_managing_plans_controller.handle_message(message, service_repo)
      elif state == States.ADMIN_VIEWING_PLAN_LIST:
        await admin_viewing_plan_list_controller.handle_message(message, service_repo)
      elif state == States.ADMIN_SETTING_PLAN_NAME:
        await admin_setting_plan_name_controller.handle_message(message, service_repo)
      elif state == States.ADMIN_SETTING_PLAN_PRICE:
        await admin_setting_plan_price_controller.handle_message(message, service_repo)

async def handle_callback_query(bot: Client, app_conf: Config, callback: CallbackQuery, state: States, service_repo: ServiceRepo, db_conf):
  if state == States.NULL:
    await null_controller.handle_callback_query(bot, callback, service_repo)
  elif state == States.STARTED:
    await started_controller.handle_callback_query(bot, callback, service_repo, db_conf, app_conf)
  elif state == States.CHOOSING_QUALITY:
    await choosing_quality_controller.handle_callback_query(bot, callback, service_repo, db_conf, app_conf)
  elif state == States.WAIT_FOR_SPOTIFY_SEARCH:
    await wait_for_spotify_search_controller.handle_callback_query(bot, callback, service_repo)
  elif state == States.CHOOSING_SPOTIFY_RESULT:
    await choosing_spotify_result_controller.handle_callback_query(bot, callback, service_repo, db_conf, app_conf)
  elif state == States.WAIT_FOR_MOVIE_SEARCH:
    await wait_for_movie_search_controller.handle_callback_query(bot, callback, service_repo)
  elif state == States.CHOOSING_MOVIE:
    await choosing_movie_controller.handle_callback_query(bot, callback, service_repo)
  elif state == States.CHOOSING_SEASON:
    await choosing_season_controller.handle_callback_query(bot, callback, service_repo)
  elif state == States.CHOOSING_EPISODE:
    await choosing_episode_controller.handle_callback_query(bot, callback, service_repo)
  elif state == States.VIEWING_PLAN:
    await viewing_plan_controller.handle_callback_query(callback, service_repo)
  elif state == States.CHOOSING_PLAN:
    await choosing_plan_controller.handle_callback_query(callback, service_repo)
  elif state == States.VIEWING_PLAN_INFO:
    await viewing_plan_info_controller.handle_callback_query(callback, service_repo)
  else:
    # Admin states
    # Check if user is admin
    if callback.from_user.id in app_conf.admin_user_ids:
      if state == States.ADMIN_STARTED:
        await admin_started_controller.handle_callback(callback, service_repo)
      elif state == States.ADMIN_WATCHING_STATS:
        await admin_watching_stats_controller.handle_callback(callback, service_repo)
      elif state == States.ADMIN_BROADCASTING_MESSAGE:
        await admin_broadcasting_message_controller.handle_callback(callback, service_repo)
      elif state == States.ADMIN_MANAGING_PLANS:
        await admin_managing_plans_controller.handle_callback(callback, service_repo)
      elif state == States.ADMIN_VIEWING_PLAN_LIST:
        await admin_viewing_plan_list_controller.handle_callback(callback, service_repo)
      elif state == States.ADMIN_SETTING_PLAN_NAME:
        await admin_setting_plan_name_controller.handle_callback(callback, service_repo)
      elif state == States.ADMIN_SETTING_PLAN_PRICE:
        await admin_setting_plan_price_controller.handle_callback(callback, service_repo)