from pyrogram.client import Client
from pyrogram.types import Message, InlineKeyboardButton, InlineKeyboardMarkup
import concurrent.futures
import json

from services.repo import ServiceRepo
from common.states import States
from common.config import Config
from common.quality import Quality
from data.main_menu import main_menu

async def handle_message(bot: Client, message: Message, service_repo: ServiceRepo, db_conf, app_conf: Config):
  user_id = message.from_user.id
  if message.text.startswith("/start spot_"):
    id = message.text.split("_")[1]
    quality = service_repo.user_service.get_quality(user_id)
    plan = service_repo.plan_member_service.get_user_plan(user_id)
    if not(user_id in app_conf.admin_user_ids) and db_conf["plans_enabled"] and quality == Quality.FLAC and not plan:
      await message.reply_text(
        text="برای دانلود با بالاترین کیفیت، اشتراک ربات را خریداری کنید. تنظیمات کیفیت دانلود به MP3 320 تغییر یافت"
      )
      quality = Quality.MP3_320
      service_repo.user_service.set_quality(user_id, Quality.MP3_320)

    await service_repo.spotify_service.spotify_dl(
      bot,
      message,
      id,
      None,
      quality,
      service_repo.user_service,
      service_repo.music_service
    )
    service_repo.user_service.set_state(user_id, States.STARTED)
    await bot.send_message(
      chat_id=message.chat.id,
      text=main_menu["text"],
      reply_markup=main_menu["reply_markup"]
    )
    return

  try:
    id = int(message.text[7:])
  except:
    return

  # Find user and set state
  service_repo.user_service.set_state(user_id, States.CHOOSING_MOVIE)

  try:
    movie = service_repo.movie_service.get_movie_by_id(id)
    if movie["songs"] and movie["is_series"] != None:
      if movie["is_series"]:
        songs = json.loads(movie["songs"])
        season_count = songs["season_count"]
        reply_markup = []
        for i in range(season_count):
          reply_markup.append([InlineKeyboardButton(
            text=f'Season {i + 1}',
            callback_data=f'dl_season_{movie["movie_name"]}:{i + 1}:{songs["seasons"][i]["episode_count"]}'
          )])
        reply_markup.append([InlineKeyboardButton(text='بازگشت', callback_data='back')])
        service_repo.user_service.set_state(user_id, States.CHOOSING_SEASON)
        await bot.send_message(
          chat_id=message.chat.id,
          text=f"""
[🎬 {movie["movie_name"]}](https://t.me/gemostbot?start={movie["id"]})

یک فصل را انتخاب کنید
""",
          reply_markup=InlineKeyboardMarkup(reply_markup)
        )
        return
      else:
        songs = json.loads(movie["songs"])
        reply_markup = []
        for song in songs:
          reply_markup.append([InlineKeyboardButton(
            text=f'{song["name"]} - {song["artist"]}',
            callback_data=f'spotsearch_{song["spot_id"]}'
          )])
        reply_markup.append([InlineKeyboardButton(text='بازگشت', callback_data='back')])
        service_repo.user_service.set_state(user_id, States.CHOOSING_SPOTIFY_RESULT)
        await bot.send_message(
          chat_id=message.chat.id,
          text=f"""
[🎬 {movie["movie_name"]}](https://t.me/gemostbot?start={movie["id"]})

آهنگ مورد نظر جهت دانلود را انتخاب کنید
""",
          reply_markup=InlineKeyboardMarkup(reply_markup)
        )
        return

    ### Not found in cache
    url = movie["tunefind_url"]
    is_series = True
    if url.split('/')[3] == 'movie':
      is_series = False

    with concurrent.futures.ThreadPoolExecutor() as executor:
      if is_series:
        future = executor.submit(service_repo.movie_service.get_show_info, url)
        result = future.result(timeout=30)
        if not result or not len(result):
          service_repo.user_service.set_state(user_id, States.STARTED)
          await bot.send_message(
            chat_id=message.chat.id, text=f'متاسفانه آهنگی برای فیلم انتخاب شده پیدا نشد\n{main_menu["text"]}', reply_markup=main_menu["reply_markup"]
          )
          return

        reply_markup = []
        cache_data = {
          "season_count": len(result),
          "seasons": []
        }
        for i in range(len(result)):
          cache_data["seasons"].append({
            "episode_count": result[i]["episodes"],
            "episodes": [[]] * int(result[i]["episodes"])
          })
          reply_markup.append([
            InlineKeyboardButton(text=result[i]["season"], callback_data=f'dl_season_{movie["movie_name"]}:{i + 1}:{result[i]["episodes"]}')
          ])
        reply_markup.append([InlineKeyboardButton(text='بازگشت', callback_data='back')])
        service_repo.movie_service.update_movie(movie["id"], True, json.dumps(cache_data))
        service_repo.user_service.set_state(user_id, States.CHOOSING_SEASON)
        await bot.send_message(
          chat_id=message.chat.id,
          text=f"""
[🎬 {movie["movie_name"]}](https://t.me/gemostbot?start={movie["id"]})

یک فصل را انتخاب کنید
""",
          reply_markup=InlineKeyboardMarkup(reply_markup)
        )

      else:
        future = executor.submit(service_repo.movie_service.get_movie_info, url)
        result = future.result(timeout=30)
        if not result or not len(result):
          service_repo.user_service.set_state(user_id, States.STARTED)
          await bot.send_message(
            chat_id=message.chat.id, text=f'متاسفانه آهنگی برای فیلم انتخاب شده پیدا نشد\n{main_menu["text"]}', reply_markup=main_menu["reply_markup"]
          )
          return
        songs = []
        for song in result:
          spot_info = await service_repo.spotify_service.base_spotify_search(
            f'track:{song["name"]} artist:{song["artist"]}',
            1
          )
          if not spot_info or not spot_info["tracks"] or not spot_info["tracks"]["items"] or not len(spot_info["tracks"]["items"]):
            continue
          spot_id = spot_info["tracks"]["items"][0]["id"]
          songs.append({
            "name": song["name"],
            "artist": song["artist"],
            "spot_id": spot_id
          })

        service_repo.movie_service.update_movie(
          movie["id"],
          False,
          json.dumps(songs)
        )

        reply_markup = []
        for song in songs:
          reply_markup.append([InlineKeyboardButton(
            text=f'{song["name"]} - {song["artist"]}',
            callback_data=f'spotsearch_{song["spot_id"]}'
          )])
        reply_markup.append([InlineKeyboardButton(text='بازگشت', callback_data='back')])
        service_repo.user_service.set_state(user_id, States.CHOOSING_SPOTIFY_RESULT)
        await bot.send_message(
          chat_id=message.chat.id,
          text=f"""
[🎬 {movie["movie_name"]}](https://t.me/gemostbot?start={movie["id"]})

آهنگ مورد نظر جهت دانلود را انتخاب کنید
""",
          reply_markup=InlineKeyboardMarkup(reply_markup)
        )

  except Exception as e:
    print(e)
    service_repo.user_service.set_state(user_id, States.STARTED)
    await bot.send_message(
      chat_id=message.chat.id, text=f'خطایی رخ داده است، دوباره امتحان کنید\n{main_menu["text"]}', reply_markup=main_menu["reply_markup"]
    )
    return
