from pyrogram.types import Message, InlineKeyboardMarkup, InlineKeyboardButton, CallbackQuery
from pyrogram.client import Client
from services.repo import ServiceRepo
from common.states import States
from common.config import Config
from common.quality import Quality
from data.main_menu import main_menu
from data.choose_quality_menu import choose_quality_menu
from data.search_movie_dialog import search_movie_dialog
from data.enter_spotify_search_term_dialog import enter_spotify_search_term_dialog
from controllers import wait_for_spotify_search as wait_for_spotify_search_controller

async def handle_message(bot: Client, message: Message, service_repo: ServiceRepo, db_conf, app_conf: Config):
  await wait_for_spotify_search_controller.handle_message(bot, message, service_repo, db_conf, app_conf)

async def handle_callback_query(bot: Client, callback: CallbackQuery, service_repo: ServiceRepo, db_conf, app_conf: Config):
  if callback.data == 'settings_init':
    service_repo.user_service.set_state(callback.from_user.id, States.CHOOSING_QUALITY)
    await bot.edit_message_text(
      chat_id=callback.from_user.id,
      message_id=callback.message.id,
      text=choose_quality_menu["text"],
      reply_markup=choose_quality_menu["reply_markup"]
    )

  elif callback.data == 'dl_spotify_init':  
    service_repo.user_service.set_state(callback.from_user.id, States.WAIT_FOR_SPOTIFY_SEARCH)
    await bot.edit_message_text(
      chat_id=callback.from_user.id,
      message_id=callback.message.id,
      text=enter_spotify_search_term_dialog["text"],
      reply_markup=enter_spotify_search_term_dialog["reply_markup"]
    )

  elif callback.data == 'find_movie_init':
    service_repo.user_service.set_state(callback.from_user.id, States.WAIT_FOR_MOVIE_SEARCH)
    await bot.edit_message_text(
      chat_id=callback.from_user.id,
      message_id=callback.message.id,
      text=search_movie_dialog["text"],
      reply_markup=search_movie_dialog["reply_markup"]
    )

  elif callback.data.startswith('spotsearch_'):
    _, spotid = callback.data.split('_')
    quality = service_repo.user_service.get_quality(callback.from_user.id)
    plan = service_repo.plan_member_service.get_user_plan(callback.from_user.id)
    if not(callback.from_user.id in app_conf.admin_user_ids) and db_conf["plans_enabled"] and quality == Quality.FLAC and not plan:
      await callback.message.reply_text(
        text="برای دانلود با بالاترین کیفیت، اشتراک ربات را خریداری کنید. تنظیمات کیفیت دانلود به MP3 320 تغییر یافت"
      )
      quality = Quality.MP3_320
      service_repo.user_service.set_quality(callback.from_user.id, Quality.MP3_320)

    await service_repo.spotify_service.spotify_dl(
      bot,
      callback.message,
      spotid,
      None,
      quality,
      service_repo.user_service,
      service_repo.music_service
    )

  elif callback.data == 'plan_management':
    # get current plan
    plan = service_repo.plan_member_service.get_user_plan(callback.from_user.id)
    if not plan:
      await callback.message.edit_text(
        text="در حال حاضر پلن فعالی ندارید",
        reply_markup=InlineKeyboardMarkup([[
          InlineKeyboardButton(text="خرید پلن", callback_data="buy"),
          InlineKeyboardButton(text="بازگشت", callback_data="back"),
        ]])
      )
    else:
      await callback.message.edit_text(
        text=f"""
پلن فعلی شما:
نام پلن: {plan["name"]}
روزهای باقی‌مانده: {plan["remaining"]}
        """,
        reply_markup=InlineKeyboardMarkup([[
          InlineKeyboardButton(text="بازگشت", callback_data="back")
        ]])
      )
    service_repo.user_service.set_state(
      callback.from_user.id,
      States.VIEWING_PLAN
    )
