from pyrogram.client import Client
from pyrogram.types import Message, CallbackQuery, InlineKeyboardButton, InlineKeyboardMarkup

from services.repo import ServiceRepo
from common.states import States
from data.main_menu import main_menu

async def handle_message(bot: Client, message: Message, service_repo: ServiceRepo):
  if message.text == '/start':
    service_repo.user_service.set_state(message.from_user.id, States.STARTED)
    await bot.send_message(
      chat_id=message.chat.id, text=main_menu["text"], reply_markup=main_menu["reply_markup"]
    )
    return
  
  movies = service_repo.movie_service.find_movies(message.text)
  if not movies or not len(movies):
    service_repo.user_service.set_state(message.from_user.id, States.CHOOSING_MOVIE)
    await bot.send_message(
      chat_id=message.chat.id,
      text=f'متاسفانه نتیجه‌ای پیدا نشد، می‌توانید درخواست جستجو در سرویس‌های دیگر را بدهید.',
      reply_markup=InlineKeyboardMarkup([[
        InlineKeyboardButton(text='بازگشت', callback_data='back'),
        InlineKeyboardButton(text='نتایج بیشتر', callback_data=f'more_movies|{message.text}')
      ]])
    )
    return

  service_repo.user_service.set_state(message.from_user.id, States.CHOOSING_MOVIE)
  reply_markup = []
  for movie in movies:
    reply_markup.append([
      InlineKeyboardButton(text=movie["movie_name"], callback_data=f'dl_movie_{movie["id"]}')
    ])
  reply_markup.append([
    InlineKeyboardButton(text='بازگشت', callback_data='back'),
    InlineKeyboardButton(text='نتایج بیشتر', callback_data=f'more_movies|{message.text}')
  ])
  await bot.send_message(
    chat_id=message.from_user.id,
    text='یکی از فیلم‌ها را انتخاب کنید',
    reply_markup=InlineKeyboardMarkup(reply_markup)
  )

async def handle_callback_query(bot: Client, callback: CallbackQuery, service_repo: ServiceRepo):
  if callback.data == 'back':
    service_repo.user_service.set_state(callback.from_user.id, States.STARTED)
    await bot.edit_message_text(
      chat_id=callback.message.chat.id,
      message_id=callback.message.id,
      text=main_menu["text"],
      reply_markup=main_menu["reply_markup"]
    )