from common import config
from services.repo import ServiceRepo
import controllers.root as root_controller
import controllers.shared_link as shared_link_controller
from controllers.check_forced_sub import check_forced_sub

from pyrogram import Client
from pyrogram.types import Message, CallbackQuery, InlineQuery

from redis_rate_limit import RateLimit, TooManyRequests

def init(app_conf: config.Config, service_repo: ServiceRepo):
  bot = Client(
    "gem_ost_bot",
    api_id=app_conf.bot_api_id,
    api_hash=app_conf.bot_api_hash,
    bot_token=app_conf.bot_token
  )
  
  @bot.on_message()
  async def handle_message(client, message: Message):
    db_conf = service_repo.db_conf_service.get_conf()

    # Check forced subscription
    sub_needed = await check_forced_sub(bot, message.from_user.id, message.chat.id, db_conf, app_conf)
    if sub_needed:
      return

    if message.text.startswith("/start") and len(message.text) > 6:
      await shared_link_controller.handle_message(client, message, service_repo, db_conf, app_conf)
      return
    try:
      with RateLimit(resource='bot_users', client=message.from_user.id, max_requests=app_conf.rate_limit_count, expire=app_conf.rate_limit_expire):
        pass
    except TooManyRequests:
      await bot.send_message(
        chat_id=message.chat.id,
        text='تعداد درخواست‌های ارسالی شما بیش از حد مجاز است، پس از چند ثانیه مجددا امتحان کنید'
      )
      return
    state = service_repo.user_service.get_state(message.from_user.id)
    await root_controller.handle_message(bot, app_conf, message, state, service_repo, db_conf)

  @bot.on_callback_query()
  async def handle_callback_query(client, callback: CallbackQuery):
    db_conf = service_repo.db_conf_service.get_conf()

    # Check forced subscription
    sub_needed = await check_forced_sub(bot, callback.from_user.id, callback.message.chat.id, db_conf, app_conf)
    if sub_needed:
      return

    if callback.data == "subscribed":
      await bot.delete_messages(
        chat_id=callback.message.chat.id,
        message_ids=[callback.message.id]
      )

    try:
      with RateLimit(resource='bot_users', client=callback.from_user.id, max_requests=app_conf.rate_limit_count, expire=app_conf.rate_limit_expire):
        pass
    except TooManyRequests:
      await bot.send_message(
        chat_id=callback.message.chat.id,
        text='تعداد درخواست‌های ارسالی شما بیش از حد مجاز است، پس از چند ثانیه مجددا امتحان کنید'
      )
      return
    state = service_repo.user_service.get_state(callback.from_user.id)
    await root_controller.handle_callback_query(bot, app_conf, callback, state, service_repo, db_conf)

  bot.run()
