from selenium import webdriver
from selenium.webdriver.common.by import By
from bs4 import BeautifulSoup
from time import sleep, time
from math import floor

from common.db_manager import DbManager

class MovieService:
  def __init__(self, db_manager: DbManager):
    self.db_manager = db_manager

  def find_movies(self, name):
    db = self.db_manager.get_connection()
    cursor = db.cursor()
    query = f'SELECT movie_name, tunefind_url, is_series, songs, fetch_date, lock_date, id FROM movie WHERE movie_name like "%{name}%" AND tunefind_url IS NOT NULL LIMIT 9;'
    cursor.execute(query)
    movies = cursor.fetchall()
    db.close()
    if not movies:
      return []
    
    result = []
    for movie in movies:
      result.append({
        "movie_name": movie[0],
        "tunefind_url": movie[1],
        "is_series": movie[2],
        "songs": movie[3],
        "fetch_date": movie[4],
        "lock_date": movie[5],
        "id": movie[6]
      })
    return result

  def get_movie(self, name):
    db = self.db_manager.get_connection()
    cursor = db.cursor()
    query = f'SELECT id, movie_name, tunefind_url, is_series, songs, fetch_date, lock_date FROM movie WHERE movie_name = "{name}";'
    cursor.execute(query)
    movie = cursor.fetchall()
    db.close()
    if not movie:
      return None
    
    return {
      "id": movie[0][0],
      "movie_name": movie[0][1],
      "tunefind_url": movie[0][2],
      "is_series": movie[0][3],
      "songs": movie[0][4],
      "fetch_date": movie[0][5],
      "lock_date": movie[0][6]
    }

  def get_movie_by_id(self, id):
    db = self.db_manager.get_connection()
    cursor = db.cursor()
    query = f'SELECT id, movie_name, tunefind_url, is_series, songs, fetch_date, lock_date FROM movie WHERE id = "{id}";'
    cursor.execute(query)
    movie = cursor.fetchall()
    db.close()
    if not movie:
      return None
    
    return {
      "id": movie[0][0],
      "movie_name": movie[0][1],
      "tunefind_url": movie[0][2],
      "is_series": movie[0][3],
      "songs": movie[0][4],
      "fetch_date": movie[0][5],
      "lock_date": movie[0][6]
    }

  def get_movie_by_id(self, id):
    db = self.db_manager.get_connection()
    cursor = db.cursor()
    query = f'SELECT id, movie_name, tunefind_url, is_series, songs, fetch_date, lock_date FROM movie WHERE id = {id};'
    cursor.execute(query)
    movie = cursor.fetchall()
    db.close()
    if not movie:
      return None
    
    return {
      "id": movie[0][0],
      "movie_name": movie[0][1],
      "tunefind_url": movie[0][2],
      "is_series": movie[0][3],
      "songs": movie[0][4],
      "fetch_date": movie[0][5],
      "lock_date": movie[0][6]
    }

  def update_movie(self, id, is_series, songs):
    db = self.db_manager.get_connection()
    cursor = db.cursor()
    cursor.execute('UPDATE movie SET is_series = %s, songs = %s WHERE id = %s;',
                   (is_series, songs, id))
    db.commit()
    db.close()

  def update_fetch_date(self, id):
    db = self.db_manager.get_connection()
    cursor = db.cursor()
    now = floor(time())
    cursor.execute(f"UPDATE movie SET fetch_date = {now} WHERE id = {id};")
    db.commit()
    db.close()

  def search_tunefind_movie(self, name):
    try:
      options = webdriver.ChromeOptions()
      options.add_argument('--headless')
      options.add_argument("start-maximized")
      options.binary_location = "/usr/bin/google-chrome-stable"
      driver = webdriver.Chrome(options=options)
      driver.execute_cdp_cmd('Network.setUserAgentOverride', {'userAgent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/117.0.0.0 Safari/537.36'})
      driver.set_window_size(1024, 768)
      driver.set_page_load_timeout(30)

      url = f"https://www.tunefind.com/search?q={name}"
      driver.get(url)
      
      movies = driver.find_elements(
        By.XPATH, '//a[starts-with(@href, "/movie/")]'
      )
      series = driver.find_elements(
        By.XPATH, '//a[starts-with(@href, "/show/")]'
      )
      
      result = []
      if movies and len(movies):
        for movie in movies:
          title = movie.find_element(
            By.TAG_NAME, 'h4'
          )
          if not title:
            continue
          title = title.text
          if not title or not len(title):
            continue
          result.append({ "title": title, "url": movie.get_attribute("href") })

      if series and len(series):
        for serie in series:
          title = serie.find_element(
            By.TAG_NAME, 'h4'
          )
          if not title:
            continue
          title = title.text
          if not title or not len(title):
            continue
          result.append({ "title": title, "url": serie.get_attribute("href") })

      driver.close()
      return result
    except:
      return None

  def get_movie_info(self, url):
    options = webdriver.ChromeOptions()
    options.add_argument('--headless')
    options.add_argument("start-maximized")
    options.binary_location = "/usr/bin/google-chrome-stable"
    driver = webdriver.Chrome(options=options)
    driver.execute_cdp_cmd('Network.setUserAgentOverride', {'userAgent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/117.0.0.0 Safari/537.36'})
    driver.set_window_size(1024, 768)
    driver.set_page_load_timeout(30)
    
    try:
      driver.get(url)
      sleep(1)
      retry_count = 0
      while retry_count < 10:
        try:
          names = driver.find_elements(
            By.XPATH, '//div[@data-rbd-droppable-id="table-body"]//descendant::p')
          artists = driver.find_elements(
            By.XPATH, '//div[@data-rbd-droppable-id="table-body"]//descendant::a/small')
          sleep(1)
          if len(names) > 0:
            break
        except:
          pass
        retry_count += 1
      
      try:
        element = driver.find_element(
          By.XPATH, '//button[contains(@class,"ShowMore")]//p[text()="Show all tracks"]/parent::button')
        driver.execute_script(
          "arguments[0].scrollIntoView({block: 'center'});", element)
        element.click()
        sleep(1)
      except:
        pass
      
      retry_count = 0
      while retry_count < 10:
        try:
          names = driver.find_elements(
            By.XPATH, '//div[@data-rbd-droppable-id="table-body"]//descendant::p')
          artists = driver.find_elements(
            By.XPATH, '//div[@data-rbd-droppable-id="table-body"]//descendant::a/small')
          sleep(1)
          if len(names) > 0:
            break
        except:
          pass
        retry_count += 1

      result = []
      for artist in artists:
        result.append({ "artist": artist.text })
      for i in range(len(names)):
        result[i]["name"] = names[i].text
      driver.close()
      return result
    except Exception as e:
      print(e)
      return None

  def get_show_info(self, url):
    options = webdriver.ChromeOptions()
    options.add_argument('--headless')
    options.add_argument("start-maximized")
    options.binary_location = "/usr/bin/google-chrome-stable"
    driver = webdriver.Chrome(options=options)
    driver.execute_cdp_cmd('Network.setUserAgentOverride', {'userAgent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/117.0.0.0 Safari/537.36'})
    driver.set_window_size(1024, 768)
    driver.set_page_load_timeout(30)

    driver.get(url)
    sleep(1)
    try:
      driver.find_element(
        By.XPATH, '//*[@id="qc-cmp2-ui"]/div[2]/div/button[2]').click()
    except:
      pass
    sleep(1)
    html = driver.page_source
    soup = BeautifulSoup(html, 'html.parser')
    datas = soup.find_all("div", {"class": "ant-row ant-row-space-between"})
    season = datas[0].div.h4.getText()

    result = []
    for data in datas:
      try:
        season = data.div.h4.getText()
        result.append({
          "season": season,
          "episodes": int(data.p.getText().split(' ')[0])
        })
      except:
          pass

    return result

  def get_season(self, url, episode):
    options = webdriver.ChromeOptions()
    options.add_argument('--headless')
    options.add_argument("start-maximized")
    options.binary_location = "/usr/bin/google-chrome-stable"
    driver = webdriver.Chrome(options=options)
    driver.execute_cdp_cmd('Network.setUserAgentOverride', {'userAgent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/117.0.0.0 Safari/537.36'})
    driver.set_window_size(1024, 768)
    driver.set_page_load_timeout(60)

    driver.get(url)
    sleep(1)

    while True:
      try:
        element = driver.find_elements(
          By.XPATH, '//div//descendant::h4[starts-with(text(),"E")]')[int(episode) - 1]
        driver.execute_script(
          "arguments[0].scrollIntoView({block: 'center'});", element)
        element.click()
        sleep(2)
        break
      except Exception as e:
        print(e)
        break

    retry_count = 0
    while True:
      retry_count = retry_count + 1
      if retry_count > 10:
        break
      try:
        names = driver.find_elements(
          By.XPATH, '//div[@data-rbd-droppable-id="table-body"]//descendant::p')
        artists = driver.find_elements(
          By.XPATH, '//div[@data-rbd-droppable-id="table-body"]//descendant::a/small')
        sleep(1)
        if len(names) > 0:
          break
      except:
        pass
    
    try:
      element = driver.find_element(
        By.XPATH, '//button[contains(@class,"ShowMore")]//p[text()="Show all tracks"]/parent::button')
      driver.execute_script(
        "arguments[0].scrollIntoView({block: 'center'});", element)
      element.click()
      sleep(1)
    except:
      pass

    retry_count = 0
    while True:
      retry_count = retry_count + 1
      if retry_count > 10:
        break
      try:
        names = driver.find_elements(
          By.XPATH, '//div[@data-rbd-droppable-id="table-body"]//descendant::p')
        artists = driver.find_elements(
          By.XPATH, '//div[@data-rbd-droppable-id="table-body"]//descendant::a/small')
        sleep(1)
        if len(names) > 0:
          break
      except:
        pass

    sleep(1)

    if not artists or not len(artists):
      return None
    result = []
    for artist in artists:
      result.append({ "artist": artist.text })
    for i in range(len(names)):
      result[i]["name"] = names[i].text

    driver.close()
    return result

  def create_movie_if_not_exists(self, title, url):
    db = self.db_manager.get_connection()
    cursor = db.cursor()
    query = 'SELECT movie_name, tunefind_url, is_series, songs, fetch_date, id FROM movie WHERE movie_name = %s'
    params = (title,)
    cursor.execute(query, params)
    movies = cursor.fetchall()
    
    if not movies or not len(movies):
      cursor.execute(
        'INSERT INTO movie (movie_name, tunefind_url) VALUES (%s, %s)',
        (title, url)
      )
      db.commit()
    else:
      if not movies[0][1]:
        cursor.execute(
          'UPDATE movie SET tunefind_url = %s WHERE id = %s',
          (url, movies[0][5])
        )
        db.commit()
    db.close()

  def get_movie_to_fetch(self):
    db =self.db_manager.get_connection()
    cursor = db.cursor()

    now =  floor(time())
    lock_ttl = 30
    lock_exp_date = now - lock_ttl

    query = f"SELECT id, movie_name, tunefind_url FROM movie WHERE fetch_date IS NULL AND (lock_date IS NULL OR lock_date < {lock_exp_date}) ORDER BY ID LIMIT 1"
    cursor.execute(query)
    movie = cursor.fetchall()
    if not movie or not len(movie):
      ttl = 432000
      exp_date = now - ttl
      query = f"SELECT id, movie_name, tunefind_url FROM movie WHERE fetch_date < {exp_date} AND (lock_date IS NULL OR lock_date < {lock_exp_date}) ORDER BY ID LIMIT 1"
      cursor.execute()
      movie = cursor.fetchall()
    db.close()
    if not movie or not len(movie):
      return None
    return {
      "id": movie[0][0],
      "movie_name": movie[0][1],
      "tunefind_url": movie[0][2]
    }
  def lock_movie(self, id):
    db = self.db_manager.get_connection()
    cursor = db.cursor()
    now =  floor(time())
    query = f"UPDATE movie SET lock_date = {now} WHERE id = {id}"
    cursor.execute(query)
    db.commit()
    db.close()

  def clear_songs(self, id):
    db = self.db_manager.get_connection()
    cursor = db.cursor()
    cursor.execute(f"UPDATE movie SET fetch_date = NULL, lock_date = NULL, songs = NULL where id = {id}")
    db.commit()
    db.close()

  def set_fetched(self, id):
    db = self.db_manager.get_connection()
    cursor = db.cursor()
    now =  floor(time())
    cursor.execute(f"UPDATE movie SET fetch_date = {now} WHERE id = {id}")
    db.commit()
    db.close()
