from pyrogram.client import Client
from pyrogram.types import Message, InlineKeyboardButton, InlineKeyboardMarkup
from pyrogram.enums import ParseMode, ChatAction
from pyrogram.errors.rpc_error import RPCError
from pyrogram.errors import FloodWait,Forbidden,UserIsBlocked,MessageNotModified,ChatWriteForbidden, SlowmodeWait 

from requests import get, head
from requests.exceptions import MissingSchema
import subprocess
from os import remove, mkdir
import os
from random import randint
from spotipy import Spotify
from yt_dlp import YoutubeDL
from asyncio import sleep
from shutil import rmtree

from mutagen import File
from mutagen.flac import FLAC ,Picture
from mutagen.mp3 import MP3
from mutagen import id3

from services.user import UserService
from services.music import MusicService
from common.states import States
from data.main_menu import main_menu
from common.config import Config
from common.quality import Quality

class SpotifyService:
  def __init__(self, client: Spotify, config: Config):
    self.client = client
    self.config = config

  async def parse_spotify_url(self, url):
    if url.startswith("spotify"):
      return url.split(":")[1]
    url = get(url).url
    parsed_url = url.replace("https://open.spotify.com/", "").split("/")
    return parsed_url[0],parsed_url[1].split("?")[0]

  async def fetch_spotify_track(self, item_id):
    item = self.client.track(track_id=item_id)
    track_name = item.get("name")
    album_info = item.get("album")
    track_artist = ", ".join([artist['name'] for artist in item['artists']])
    if album_info:
      track_album = album_info.get('name')
      track_year = album_info.get('release_date')[:4] if album_info.get('release_date') else ''
      album_total = album_info.get('total_tracks')
    track_num = item['track_number']
    deezer_id = item_id
    cover = item['album']['images'][0]['url'] if len(item['album']['images']) > 0 else None
    genre = self.client.artist(artist_id=item['artists'][0]['uri'])['genres'][0] if len(self.client.artist(artist_id=item['artists'][0]['uri'])['genres']) > 0 else ""
    offset = 0
    return {
      "name": track_name,
      "artist": track_artist,
      "album": track_album,
      "year": track_year,
      "num_tracks": album_total,
      "num": track_num,
      "playlist_num": offset + 1,
      "cover": cover,
      "genre": genre,
      "deezer_id": deezer_id,
    }

  async def getIds(self, video):
    ids = []
    with YoutubeDL({'quiet':True}) as ydl:
      info_dict = ydl.extract_info(video, download=False)
      try:
        info_dict = info_dict['entries']
        ids.extend([x.get('id'),x.get('playlist_index'),x.get('creator') or x.get('uploader'),x.get('title'),x.get('duration'),x.get('thumbnail')] for x in info_dict)
      except:
        ids.append([info_dict.get('id'),info_dict.get('playlist_index'),info_dict.get('creator') or info_dict.get('uploader'),info_dict.get('title'),info_dict.get('duration'),info_dict.get('thumbnail')])
    return ids

  async def download_songs(self, item, download_directory, codec, bitrate):
    # For mp3 - 320k, download the flac file and convert it
    if codec == 'mp3' and bitrate == '320K':
        newcodec = 'flac'
        newbitrate = '693'
    else:
        newcodec = codec
        newbitrate = bitrate

    file = f"{download_directory}/{item['name']} - {item['artists'][0]['name']}"
    query = f"{item['name']} - {item['artists'][0]['name']}".replace(":", "").replace("\"", "")
    ydl_opts = {
        'format': "bestaudio",
        'default_search': 'ytsearch',
        'noplaylist': True,
        "nocheckcertificate": True,
        "outtmpl": file,
        "quiet": True,
        "addmetadata": True,
        "prefer_ffmpeg": False,
        "geo_bypass": True,

        "nocheckcertificate": True,
        "postprocessors": [{'key': 'FFmpegExtractAudio', 'preferredcodec': newcodec, 'preferredquality': newbitrate}],
    }

    with YoutubeDL(ydl_opts) as ydl:
        try:
            video = ydl.extract_info(f"ytsearch:{query}", download=False)['entries'][0]['id']
            info = ydl.extract_info(video)
            filename = ydl.prepare_filename(info)
            # return f"{filename}.{codec}"
        except IndexError:
            pass
            quer = f"{item['name']} lyrics"
            video = ydl.extract_info(f"ytsearch:{quer}", download=False)['entries'][0]['id']
            info = ydl.extract_info(video)
            filename = ydl.prepare_filename(info)
            # return f"{filename}.{codec}" 
        except (IOError,BrokenPipeError):
            pass
            video = ydl.extract_info(f"ytsearch:{query}", download=False)['entries'][0]['id']
            info = ydl.extract_info(video)
            filename = ydl.prepare_filename(info)
            # return f"{filename}.{codec}"
        except Exception as e:
            print(e)

    # Convert flac to mp3 if needed
    if codec == 'mp3' and bitrate == '320K':
        p = subprocess.Popen(
            f"ffmpeg -i \"{filename}.{newcodec}\" -ab 320k -hide_banner -loglevel error -map_metadata 0 -id3v2_version 3 \"{filename}.{codec}\"",
            shell=True, stdout=subprocess.PIPE
        )
        p.wait()
        remove(f"{filename}.{newcodec}")
    return f"{filename}.{codec}"

  async def thumb_down(self, link,deezer_id):
    tmp_dir = self.config.tmp_dir
    with open(os.path.join(tmp_dir, 'thumbnails', f'{deezer_id}.jpg'), 'wb') as file:
      file.write(get(link).content)
    return os.path.join(tmp_dir, 'thumbnails', f'{deezer_id}.jpg')

  async def base_spotify_search(self, text, limit):
    print(f"Searching spotify for {text}")
    results = self.client.search(text, limit)
    return results

  async def spotify_dl(self, bot: Client, message: Message, id, link, quality: Quality, user_service: UserService, music_service: MusicService):
    if not quality:
      quality = Quality.MP3_320
    if quality == Quality.FLAC:
      codec = 'flac'
      bitrate = '693'
    else:
      codec, bitrate = quality.name.split('_')
      codec = codec.lower()
      bitrate = f"{bitrate}K"

    back_button = InlineKeyboardMarkup([[InlineKeyboardButton(text="بازگشت", callback_data="cancel")]])
    
    print(f"Starting download: {link}, {id}")
    if link:
        # Process the link
      if "https://spotify.link" in link:
        link = head(link).headers['location']
      try:
        parsed_item = await self.parse_spotify_url(link)
        item_type, item_id = parsed_item[0],parsed_item[1]
      except Exception as e:
        try:
          link = head(link).headers['location']
          parsed_item = await self.parse_spotify_url(link)
          item_type, item_id = parsed_item[0],parsed_item[1]
        except Exception as e:
          await bot.send_message(
            chat_id=message.from_id.id,
            text=f"""
متاسفانه خطایی پیش آمد، دوباره امتحان کنید.
{main_menu["text"]}
""",
            reply_markup=main_menu["reply_markup"]
          )
          user_service.set_state(message.from_user.id, States.STARTED)
          return
    else:
      item_type = "track"
      item_id = id

    uploaded = music_service.get_music(item_id, codec, bitrate)
    if uploaded:
      item_id = uploaded["spot_id"]
      file_id = uploaded["file_id"]
      caption = uploaded["caption"]
      print(f'ID {item_id} exists with file ID {file_id}')
    else:
      file_id = None
    
    result_markup = InlineKeyboardMarkup([[
      InlineKeyboardButton(text='بازگشت', callback_data='back'),
      InlineKeyboardButton(text='متن آهنگ', callback_data=f'lyrics:{item_id}'),
    ]])

    if file_id:
      await bot.send_audio(
        chat_id=message.chat.id,
        audio=file_id,
        caption=caption,
        parse_mode=ParseMode.MARKDOWN,
        reply_markup=result_markup
      )
      user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
      return

    try: 
      if item_type in ["https:","http:"]:
        await bot.send_message(
          chat_id=message.from_id.id,
          text=f"""
متاسفانه خطایی پیش آمد، دوباره امتحان کنید.
{main_menu["text"]}
""",
          reply_markup=main_menu["reply_markup"]
        )
        user_service.set_state(message.from_user.id, States.STARTED)
        return
    except Exception as e:
      pass

    # Create temp directory and send waiting message  
    u = message.from_user.id
    randomdir = os.path.join(self.config.tmp_dir, str(randint(1, 100000000)))
    mkdir(randomdir)

    try:
      if item_type in ["show", "episode"]:
        await bot.send_message(
          chat_id=message.from_id.id,
          text=f"""
در حال حاضر دانلود کردن سریال‌ها پشتیبانی نمی‌شود. لطفا لینک آهنگ، آلبوم یا پلی‌لیست ارسال کنید.
{main_menu["text"]}
""",
          reply_markup=main_menu["reply_markup"]
        )
        user_service.set_state(message.from_user.id, States.STARTED)
        return
      
      elif item_type == "track":
        print('Fetching track')
        song = await self.fetch_spotify_track(item_id)
        print('fetched track')
        try:
          item = self.client.track(track_id=item_id)
        except:
          pass

        try:
          if not item:
            await message.reply_photo(
              song.get('cover'),
              caption=f"⬇️ آهنگ به زودی دانلود می‌شود و برای شما ارسال می‌شود\n🎧 Title : `{song['name']}`\n🎤 Artist : `{song['artist']}`\n💽 Album : `{song['album']}`\n🗓 Release Year: `{song['year']}`\n\n[IMAGE]({song.get('cover')})\nTrack id:`{song['deezer_id']}`",
              reply_markup=InlineKeyboardMarkup([[InlineKeyboardButton(text="بازگشت", callback_data="back")]])
            )
            user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
          else:
            await message.reply_photo(
              item['album']['images'][0]['url'],
              caption=f"⬇️ آهنگ به زودی دانلود می‌شود و برای شما ارسال می‌شود\n🎧 Title : `{song['name']}­­`\n🎤 Artist : `{song['artist']}`­\n💽 Album : `{song['album']}`\n🗓 Release Year: `{song['year']}`\n❗️Is Local:`{item['is_local']}`\n 🌐ISRC: `{item['external_ids']['isrc']}`\n\n[IMAGE]({item['album']['images'][0]['url']})\nTrack id:`{song['deezer_id']}`",
              reply_markup=InlineKeyboardMarkup([[InlineKeyboardButton(text="بازگشت", callback_data="back")]])
            )
            user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
        except:
          await message.reply_text(f"⬇️ آهنگ به زودی دانلود می‌شود و برای شما ارسال می‌شود\n🎧 Title : `{song['name']}`\n­🎤 Artist : `{song['artist']}`\n💽 Album : `{song['album']}`\n🗓 Release Year: `{song['year']}`\n\n[IMAGE]({song.get('cover')})\ntrack id:`{song['deezer_id']}`"
                                                ,reply_markup=InlineKeyboardMarkup([[InlineKeyboardButton(text="بازگشت", callback_data="back")]]))
          user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)

        try:
          path = await self.download_songs(item, randomdir, codec, bitrate)
        except Exception as e:
          print(e)
          await message.reply_text(f"[{song.get('name')} - {song.get('artist')}](https://open.spotify.com/track/{song.get('deezer_id')}) آهنگ پیدا نشد ⚠️",reply_markup=InlineKeyboardMarkup([[InlineKeyboardButton(text="بازگشت", callback_data="back")]]))
          user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
          return

        thumbnail = await self.thumb_down(item['album']['images'][0]['url'],song.get('deezer_id'))
        try:
          if codec == 'flac':
            audio = FLAC(path)
          else:
            audio = MP3(path)
          audio["TITLE"] = id3.TextFrame(encoding=3, text=f"{song.get('name')}") if codec == 'mp3' else f"{song.get('name')}"
          audio["ORIGINALYEAR"] = id3.TextFrame(encoding=3, text=song.get('year')) if codec == 'mp3' else song.get('year')
          audio["YEAR_OF_RELEASE"] = id3.TextFrame(encoding=3, text=song.get('year')) if codec == 'mp3' else song.get('year')
          audio["WEBSITE"] = id3.TextFrame(encoding=3, text="https://t.me/gemostbot") if codec == 'mp3' else "https://t.me/gemostbot"
          audio["GEEK_SCORE"] = id3.TextFrame(encoding=3, text="9") if codec == 'mp3' else "9"
          audio["ARTIST"] = id3.TextFrame(encoding=3, text=song.get('artist')) if codec == 'mp3' else song.get('artist')
          audio["ALBUM"] = id3.TextFrame(encoding=3, text=song.get('album')) if codec == 'mp3' else song.get('album')
          audio["DATE"] = id3.TextFrame(encoding=3, text=song.get('year')) if codec == 'mp3' else song.get('year')
          audio["DISCNUMBER"] =id3.TextFrame(encoding=3, text=f"{item['disc_number']}") if codec == 'mp3' else f"{item['disc_number']}"
          audio["TRACKNUMBER"] =id3.TextFrame(encoding=3, text=f" {item['track_number']}") if codec == 'mp3' else f" {item['track_number']}"
          try:
            audio["ISRC"] = id3.TextFrame(encoding=3, text=item['external_ids']['isrc']) if codec == 'mp3' else item['external_ids']['isrc']
          except:
            pass
          audio.save()
          audi = File(path)
          image = Picture() 
          image.type = 3
          if thumbnail.endswith('png'):
            mime = 'image/png'
          else:
            mime = 'image/jpeg'
          image.desc = 'front cover'
          with open(thumbnail, 'rb') as f: # better than open(albumart, 'rb').read() ?
            image.data = f.read()

          if codec == 'flac':
            audi.add_picture(image)
          audi.save()
        except Exception as ee:
          print(ee)
          pass
        try:
          dForChat = await message.reply_chat_action(ChatAction.UPLOAD_AUDIO)
          # caption = f"[{song.get('name')}](https://open.spotify.com/track/{song.get('deezer_id')}) | {song.get('album')} - {song.get('artist')}"
          caption = f"[{song.get('name')} - {song.get('artist')}](https://t.me/gemostbot?start=spot_{item_id})"
          performer = f"{song.get('artist')}"
          title = f"{song.get('name')} - {song.get('artist')}"
          backup_message = await bot.send_audio(chat_id=self.config.backup_channel_id, audio=path, title=title, performer=performer, thumb=thumbnail)
          music_service.save_music(item_id, backup_message.audio.file_id, codec, bitrate, caption=caption)
          await bot.send_audio(chat_id=message.chat.id, audio=backup_message.audio.file_id,caption=caption, parse_mode=ParseMode.MARKDOWN,
              reply_markup=result_markup)
          user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
        except Exception as e:
          print(e)
          pass

      elif item_type == "playlist":
        play = self.client.playlist(playlist_id=item_id,)
        tracks = self.client.playlist_items(playlist_id=item_id,additional_types=['track'], offset=0, market=None)   
        total_tracks = tracks.get('total')
        track_no = 1
        try:
          await message.reply_photo(play['images'][0]['url'],
            caption=f"⬇️ در حال دانلود پلی‌لیست\n▶️Playlist:{play['name']}\n📝Description:{play['description']}\n👤Owner:{play['owner']['display_name']}\n❤️Followers:{play['followers']['total']}\n🔢 Total Track:{play['tracks']['total']}\n\n[IMAGES]({play['images'][0]['url']})\n{play['uri']}"
            ,reply_markup=InlineKeyboardMarkup([[InlineKeyboardButton(text="بازگشت", callback_data="back")]]))
          user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
        except Exception as e:
          await bot.send_message(
            chat_id=message.from_id.id,
            text=f"""
لینک پلی‌لیست معتبر نیست.
{main_menu["text"]}
""",
            reply_markup=main_menu["reply_markup"]
          )
          user_service.set_state(message.from_user.id, States.STARTED)
          return
        
        for track in tracks['items']:
          song = await self.fetch_spotify_track(track.get('track').get('id'))
          item = self.client.track(track_id=track['track']['id'])

          item_id = track['track']['id']
          uploaded = music_service.get_music(item_id, codec, bitrate)
          if uploaded:
            item_id = uploaded["spot_id"]
            file_id = uploaded["file_id"]
            caption = uploaded["caption"]
            print(f'ID {item_id} exists with file ID {file_id}')
          else:
            file_id = None
          
          if file_id:
            await bot.send_audio(
              chat_id=message.chat.id,
              audio=file_id,
              caption=caption,
              parse_mode=ParseMode.MARKDOWN,
              reply_markup=result_markup
            )
            user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
            continue

          try:
            path = await self.download_songs(item, randomdir, codec, bitrate)
          except Exception as e:
            await bot.send_message(
              chat_id=message.from_id.id,
              text=f"""
خطایی در دانلود پلی‌لیست پیش آمد.
{main_menu["text"]}
""",
              reply_markup=main_menu["reply_markup"]
            )
            user_service.set_state(message.from_user.id, States.STARTED)
            return
            
          thumbnail = await self.thumb_down(song.get('cover'),song.get('deezer_id'))
          if codec == 'flac':
            audio = FLAC(path)
          else:
            audio = MP3(path)
          try:
            audio["TITLE"] = id3.TextFrame(encoding=3, text=f"{song.get('name')}") if codec == 'mp3' else f"{song.get('name')}"
            audio["ORIGINALYEAR"] = id3.TextFrame(encoding=3, text=song.get('year')) if codec == 'mp3' else song.get('year')
            audio["YEAR_OF_RELEASE"] = id3.TextFrame(encoding=3, text=song.get('year')) if codec == 'mp3' else song.get('year')
            audio["WEBSITE"] = id3.TextFrame(encoding=3, text="https://t.me/gemostbot") if codec == 'mp3' else "https://t.me/gemostbot"
            audio["GEEK_SCORE"] = id3.TextFrame(encoding=3, text="9") if codec == 'mp3' else "9"
            audio["ARTIST"] = id3.TextFrame(encoding=3, text=song.get('artist')) if codec == 'mp3' else song.get('artist')
            audio["ALBUM"] = id3.TextFrame(encoding=3, text=song.get('album')) if codec == 'mp3' else song.get('album')
            audio["DATE"] = id3.TextFrame(encoding=3, text=song.get('year')) if codec == 'mp3' else song.get('year')
            audio["discnumber"] =id3.TextFrame(encoding=3, text=f" {item['disc_number']}") if codec == 'mp3' else f" {item['disc_number']}"
            audio["tracknumber"] =id3.TextFrame(encoding=3, text=f" {item['track_number']}") if codec == 'mp3' else f" {item['track_number']}"
            try:
              audio["ISRC"] = id3.TextFrame(encoding=3, text=item['external_ids']['isrc']) if codec == 'mp3' else item['external_ids']['isrc']
            except:
              pass
          except:
            pass
          audio.save()
          audi = File(path)
          image = Picture()
          image.type = 3
          if thumbnail.endswith('png'):
            mime = 'image/png'
          else:
            mime = 'image/jpeg'
          image.desc = 'front cover'
          with open(thumbnail, 'rb') as f: # better than open(albumart, 'rb').read() ?
            image.data = f.read()
          if codec == 'flac':
            audi.add_picture(image)
          audi.save()
          try:
            await message.reply_chat_action(ChatAction.UPLOAD_AUDIO)
            performer = song.get('artist')
            title = f"{song.get('name')} - {song.get('artist')}"
            caption = f"[{song.get('name')} - {song.get('artist')}](https://t.me/gemostbot?start=spot_{item_id})"
            backup_message = await bot.send_audio(chat_id=self.config.backup_channel_id, audio=path,performer=performer,title=title, thumb=thumbnail)
            music_service.save_music(item_id, backup_message.audio.file_id, codec, bitrate, caption=caption)
            await bot.send_audio(chat_id=message.chat.id, audio=backup_message.audio.file_id,caption=caption,parse_mode=ParseMode.MARKDOWN,
              reply_markup=result_markup) 
            user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
          except Exception as e:
            print(e)
          track_no += 1

        await bot.send_message(
          chat_id=message.chat.id,
          text="پلی‌لیست دانلود شد",
          reply_markup=InlineKeyboardMarkup([[InlineKeyboardButton(text="بازگشت", callback_data="back")]])
        )
        user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
          
      elif item_type == "album":
        alb = self.client.album(album_id=item_id,)
        try:
          await message.reply_photo(alb['images'][0]['url'],
            caption=f"⬇️ آلبوم در حال دانلود است و به زودی ارسال می‌شود\n💽Album: {alb['name']}\n👥Artists: {alb['artists'][0]['name']}\n🎧Total tracks{alb['total_tracks']}\n🗂Category: {alb['album_type']}\n📆Published on: {alb['release_date']}\n\n[IMAGE]({alb['images'][0]['url']})\n{alb['uri']}"
            , reply_markup=InlineKeyboardMarkup([[InlineKeyboardButton(text="بازگشت", callback_data="back")]]))
          user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
        except Exception as e:
          await message.reply(f"⬇️ آلبوم در حال دانلود است و به زودی ارسال می‌شود\n💽Album: {alb['name']}\n👥Artists: {alb['artists'][0]['name']}\n🎧Total tracks{alb['total_tracks']}\n🗂Category: {alb['album_type']}\n📆Published on: {alb['release_date']}\n\n[IMAGE]({alb['images'][0]['url']})\n{alb['uri']}"
                                          , reply_markup=InlineKeyboardMarkup([[InlineKeyboardButton(text="بازگشت", callback_data="back")]]))
          user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)

        for track in alb['tracks']['items']:
          item = self.client.track(track_id=track['id'])

          item_id = track['id']
          uploaded = music_service.get_music(item_id, codec, bitrate)
          if uploaded:
            item_id = uploaded["spot_id"]
            file_id = uploaded["file_id"]
            caption = uploaded["caption"]
            print(f'ID {item_id} exists with file ID {file_id}')
          else:
            file_id = None
          
          if file_id:
            await bot.send_audio(
              chat_id=message.chat.id,
              audio=file_id,
              caption=caption,
              parse_mode=ParseMode.MARKDOWN,
              reply_markup=result_markup
            )
            user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
            continue

          song = await self.fetch_spotify_track(track.get('id'))
          try:
            path = await self.download_songs(item,randomdir, codec, bitrate)
          except Exception as e:
            await message.reply_text(f"[{song.get('name')} - {song.get('artist')}](https://open.spotify.com/track/{song.get('deezer_id')}) آهنگ پیدا نشد ⚠️", reply_markup=InlineKeyboardMarkup([[InlineKeyboardButton(text="بازگشت", callback_data="back")]]))
            user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)

          thumbnail = await self.thumb_down(song.get('cover'),song.get('deezer_id'))
          if codec == 'flac':
            audio = FLAC(path)
          else:
            audio = MP3(path)
          try:
            audio["TITLE"] = id3.TextFrame(encoding=3, text=f" {song.get('name')} ") if codec == 'mp3' else f" {song.get('name')} "
            audio["ORIGINALYEAR"] = id3.TextFrame(encoding=3, text=song.get('year')) if codec == 'mp3' else song.get('year')
            audio["YEAR_OF_RELEASE"] = id3.TextFrame(encoding=3, text=song.get('year')) if codec == 'mp3' else song.get('year')
            audio["WEBSITE"] = id3.TextFrame(encoding=3, text="https://t.me/gemostbot") if codec == 'mp3' else "https://t.me/gemostbot"
            audio["GEEK_SCORE"] = id3.TextFrame(encoding=3, text="9") if codec == 'mp3' else "9"
            audio["ARTIST"] = id3.TextFrame(encoding=3, text=song.get('artist')) if codec == 'mp3' else song.get('artist')
            audio["ALBUM"] = id3.TextFrame(encoding=3, text=song.get('album')) if codec == 'mp3' else song.get('album')
            audio["DATE"] = id3.TextFrame(encoding=3, text=song.get('year')) if codec == 'mp3' else song.get('year')
            audio["discnumber"] =id3.TextFrame(encoding=3, text=f" {item['disc_number']}") if codec == 'mp3' else f" {item['disc_number']}"
            audio["tracknumber"] =id3.TextFrame(encoding=3, text=f" {item['track_number']}") if codec == 'mp3' else f" {item['track_number']}"
            try:
                audio["ISRC"] =id3.TextFrame(encoding=3, text=f" {item['external_ids']['isrc']}") if codec == 'mp3' else f" {item['external_ids']['isrc']}"
            except:
                pass
            # try:
            #     songGenius = genius.search_song(song('name'), song('artist'))
            #     audio["LYRICS"] = (songGenius.lyrics)
            # except:
            #    pass
          except:
            pass
          audio.save()
          audi = File(path)
          image = Picture()
          image.type = 3
          if thumbnail.endswith('png'):
            mime = 'image/png'
          else:
            mime = 'image/jpeg'
          image.desc = 'front cover'
          with open(thumbnail, 'rb') as f: # better than open(albumart, 'rb').read() ?
            image.data = f.read()

          if codec == 'flac':
            audi.add_picture(image)
          audi.save()
          try:
            performer=song.get('artist')
            title=f"{song.get('name')} - {song.get('artist')}"
            caption = f"[{song.get('name')} - {song.get('artist')}](https://t.me/gemostbot?start=spot_{item_id})"
            backup_message = await bot.send_audio(chat_id=self.config.backup_channel_id, audio=path, performer=performer, title=title ,thumb=thumbnail)
            music_service.save_music(item_id, backup_message.audio.file_id, codec, bitrate, caption=caption)
            await bot.send_audio(chat_id=message.chat.id, audio=backup_message.audio.file_id,caption=caption, parse_mode=ParseMode.MARKDOWN,
                reply_markup=result_markup)
            user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
          except:
            pass

        await bot.send_message(
          chat_id=message.chat.id,
          text="آلبوم دانلود شد",
          reply_markup=InlineKeyboardMarkup([[InlineKeyboardButton(text="بازگشت", callback_data="back")]])
        )
        user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)

    except MissingSchema:
      pass
      await message.reply("آدرس اشتباه است", reply_markup=back_button)
      return
    except RPCError:
      pass
      await message.reply("از سمت تلگرام خطایی پیش آمده است، مجددا امتحان کنید", reply_markup=back_button)
      return
    except ChatWriteForbidden:
      pass
      chat=message.chat.id
      try:
        await bot.leave_chat(chat)
        k = await bot.send_message(-1001744816254,f"{chat} {message.chat.username} or {message.from_user.id}")
        await  k.pin()
        sp = f"I have left from {chat} reason: I Am Not  Admin "
        await bot.send_message(message.from_user.id,f"{sp}")
      except:
        pass
      return
    except UserIsBlocked:
      pass
      # K = await  bot.send_message(BUG,f" private {message.chat.id}  {message.from_user.id} {message.from_user.mention}")
      # k.pin()
    except IOError:
      pass
      # K = await  bot.send_message(BUG,f" Private r: socket {message.chat.id}  {message.from_user.id} {message.from_user.mention}")
      # k.pin()
    except (FileNotFoundError, OSError):
      pass
      await message.reply("خطایی پیش آمده و قادر به پردازش درخواست نیستم. به ادمین اطلاع دهید", reply_markup=back_button)
      return
    except BrokenPipeError:
      pass
      # K = await  bot.send_message(BUG,f" private r: broken {message.chat.id}  {message.from_user.id} {message.from_user.mention}")
      return
    except Forbidden:
      T = await message.reply_text("ربات اجازه انجام این عملیات را ندارد", reply_markup=back_button)
      return
    except UnboundLocalError:
      pass
        
    except FloodWait as e:
      pass
      await sleep(e.value)
      await message.reply_text(f"لطفا پس از {e.value} ثانیه مجددا امتحان کنید", reply_markup=back_button)
      return
    except SlowmodeWait:
      pass
      await sleep(e.value)
    except IOError as e:
      pass
      # K = await  bot.send_message(BUG,f" private r: broken {message.chat.id} {message.from_user.mention}")
           
    except Exception as e:
      print(e)
      pass
      await message.reply('قادر به پردازش درخواست شما نیستم، بعدا امتحان کنید', reply_markup=back_button)
    finally:
      await sleep(2.0)
      try:
        rmtree(randomdir)
      except:
        pass

  async def spotify_search(self, bot: Client, message: Message, text, message_to_delete: Message, user_service: UserService, offset: int = 0, limit: int = 10):
    if message_to_delete:
      await message_to_delete.delete()
    waiting_msg = await message.reply("⌛")
    reply_markup = []
    try:
      print(f"F Searching spotify for {text}")
      results = self.client.search(text, limit=offset + limit)
      index = 0
      print("search done")
      for result in results['tracks']['items']:
        if index >= offset:
          reply_markup.append(
              [InlineKeyboardButton(f"{result['name']} - {result['artists'][0]['name']}", callback_data=f"spotsearch_{results['tracks']['items'][int(index)]['id']}")]
          )
        index += 1
      reply_markup.append([InlineKeyboardButton("➕ نتایج بیشتر", callback_data=f"more:{offset + limit}:{limit}:{text}")])
      reply_markup.append([InlineKeyboardButton("🔙 بازگشت", callback_data="back")])
      count = len(reply_markup)

      await waiting_msg.delete()
      reply_markup=InlineKeyboardMarkup(reply_markup)
      if count == 2:
        await message.reply(
          # chat_id=message.from_user.id,
          text='متاسفانه هیچ نتیجه‌ای پیدا نشد',
          reply_markup=reply_markup
        )
      else:
        await message.reply(
          # chat_id=message.from_user.id,
          text='یکی از نتایج را انتخاب کنید',
          reply_markup=reply_markup
        )
      user_service.set_state(message.from_user.id, States.CHOOSING_SPOTIFY_RESULT)

    except Exception as e:
      print(e)
      await waiting_msg.delete()
      await message.reply(
        chat_id=message.from_user.id,
        text=f"""
متاسفانه نتیجه‌ای پیدا نشد.
{main_menu["text"]}
""",
        reply_markup=main_menu["reply_markup"]
      )
      user_service.set_state(message.from_user.id, States.STARTED)